__title__ = "dj-rest-auth-social"
__description__ = "Authentication and Registration in Django Rest Framework."
__url__ = "https://github.com/robertwt7/dj-rest-auth-social"
__version__ = "2.2.4"
__author__ = "@robertwt7 https://github.com/robertwt7"
__author_email__ = "robert@sharkware.org"
__license__ = "MIT"
__copyright__ = "Copyright 2020 @robertwt7 https://github.com/robertwt7"
